# jwt_full/jwa.nim
# Copyright 2021 Jonathan Bernard

##
## ====================================
##                jwa
## ====================================
##
## ------------------------------------
## JSON Web Algorithms (JWA) - RFC 7518
## ------------------------------------
##
## Enum values reflecting the names of algorithms defined by JWA RFC 7518.

type
  JwtKeyType* = enum ktyEC = "EC", ktyRSA = "RSA", ktyOctet = "oct"

  JwtAlgorithm* = enum
    HS256 = "HS256", HS384 = "HS384", HS512 = "HS512",
    RS256 = "RS256", RS384 = "RS384", RS512 = "RS512",
    ES256 = "ES256", ES384 = "ES384", ES512 = "ES512",
    PS256 = "PS256", PS384 = "PS384", PS512 = "PS512",

    algNone = "none", algDir = "dir",

    RSA1_5 = "RSA1_5", RSA_OAEP = "RSA-OAEP", RSA_OAEP_256 = "RSA-OEAP-256",

    A128KW = "A128KW", A192KW = "A192KW", A256KW = "A256KW",

    ECDH_ES = "ECDH_ES", ECDH_ES_A128KW = "ECDH_ES_A128KW",
    ECDH_ES_A192KW = "ECDH_ES_A192KW", ECDH_ES_A256KW = "ECDH_ES_A256KW",

    A128GCMKW = "A128GCMKW", A192GCMKW = "A192GCMKW", A256GCMKW = "A256GCMKW",

    PBES2_HS256_A128KW = "PBES2_HS256_A128KW",
    PBES2_HS384_A192KW = "PBES2_HS384_A192KW",
    PBES2_HS512_A256KW = "PBES2_HS512_A256KW",

    A128CBC_HS256 = "A128CBC_HS256",
    A192CBC_HS384 = "A192CBC_HS384",
    A256CBC_HS512 = "A256CBC_HS512",

    A128GCM = "A128GCM", A192GCM = "A192GCM", A256GCM = "A256GCM"

  EcCurve* = enum P256 = "P-256", P384 = "P-384", P521 = "P-521"
