# jwt_full/jwssig.nim
# Copyright 2021 Jonathan Bernard

## =====================================
##               JwsSig
## =====================================
##
## -------------------------------------
## JwsSignature object and procedures.
## -------------------------------------

import std/json, std/options

import ./jwa
import ./joseheader
import ./private/jsonutils

type
  JwsSignature* = object
    protected: JoseHeader
    header: JoseHeader
    signatureB64: string

func protected*(sig: JwsSignature): JoseHeader = sig.protected
func header*(sig: JwsSignature): JoseHeader = sig.header
func signatureB64*(sig: JwsSignature): string = sig.signatureB64

func initJwsSignature*(
    header: JoseHeader = initJoseHeader(alg = none[JwtAlgorithm]()),
    protected: JoseHeader,
    signatureB64: string
  ): JwsSignature =

  if not header.alg.isSome and not protected.alg.isSome:
    raise newException(ValueError,
      "alg header parameter is required but not present in either the " &
      "protected or unprotected headers.")

  return JwsSignature(
    header: header,
    protected: protected,
    signatureB64: signatureB64)

proc initJwsSignature*(n: JsonNode): JwsSignature =
  return JwsSignature(
    header: initJoseHeader(parseJson(n.reqStrVal("header"))),
    protected: initJoseHeader(n.reqStrVal("protected")),
    signatureB64: n.reqStrVal("signature"))

func `%`*(sig: JwsSignature): JsonNode =
  %*{
    "protected": sig.protected.rawB64,
    "header": sig.header.rawB64,
    "signature": sig.signatureB64
  }
