import std/tables
import bearssl

type
  HashAlgorithm* = enum SHA1, SHA256, SHA384, SHA512

  HashCfg* = object
    alg*: HashAlgorithm
    vtable*: ptr HashClass
    oid*: cstring
    size*: int

let HASHES* = newTable[HashAlgorithm, HashCfg]([
  (SHA1, HashCfg(
    alg: SHA1,
    vtable: addr sha1Vtable,
    oid: HASH_OID_SHA1,
    size: sha1Size)),
  (SHA256, HashCfg(
    alg: SHA256,
    vtable: addr sha256Vtable,
    oid: HASH_OID_SHA256,
    size: sha256Size)),
  (SHA384, HashCfg(
    alg: SHA384,
    vtable: addr sha384Vtable,
    oid: HASH_OID_SHA384,
    size: sha384Size)),
  (SHA512, HashCfg(
    alg: SHA512,
    vtable: addr sha512Vtable,
    oid: HASH_OID_SHA512,
    size: sha512Size)),
])

proc hashConfig*(alg: HashAlgorithm): HashCfg =
  case alg:
    of SHA1: return HashCfg(
      alg: SHA1,
      vtable: addr sha1Vtable,
      oid: HASH_OID_SHA1,
      size: sha1Size)

    of SHA256: return HashCfg(
      alg: SHA256,
      vtable: addr sha256Vtable,
      oid: HASH_OID_SHA256,
      size: sha256Size)

    of SHA384: return HashCfg(
      alg: SHA384,
      vtable: addr sha384Vtable,
      oid: HASH_OID_SHA384,
      size: sha384Size)

    of SHA512: return HashCfg(
      alg: SHA512,
      vtable: addr sha512Vtable,
      oid: HASH_OID_SHA512,
      size: sha512Size)

proc hash*(data: string, alg: HashAlgorithm): string =

  var ctx: HashCompatContext
  var pCtx = cast[ptr ptr HashClass](addr ctx)
  let hashCfg = hashConfig(alg)
  result = newString(hashCfg.size)
  hashCfg.vtable.init(pCtx)
  hashCfg.vtable.update(pCtx, data.cstring, uint data.len)
  hashCfg.vtable.out(pCtx, addr result[0])
