import bearssl

# Taken from  nim-bearssl/decls.nim
{.pragma: bearSslFunc, cdecl, gcsafe, noSideEffect, raises: [].}

type BearPemDecoderCallback* = proc(destCtx: pointer, src: pointer, dataLen: uint) {.bearSslFunc.}

proc decodePem*(
    pem: string,
    expectedObjectName: string,
    keyCtx: pointer,
    callback: BearPemDecoderCallback
  ) =

  var pemCtx: PemDecoderContext
  pemDecoderInit(pemCtx)

  var offset: uint = 0
  var bytesRemaining: uint = uint len(pem)
  var readingObj = false

  while bytesRemaining > 0:
    let bytesRead = pemDecoderPush(pemCtx, unsafeAddr pem[offset], uint bytesRemaining)
    offset += bytesRead
    bytesRemaining -= bytesRead

    case pemDecoderEvent(pemCtx):
      of PEM_BEGIN_OBJ:
        if readingObj:
          raise newException(ValueError,
            "Invalid PEM: saw a second BEGIN before seeing END.")

        if pemDecoderName(pemCtx) != expectedObjectName:
          raise newException(ValueError,
            "Invalid PEM: expected BEGIN " & expectedObjectName &
            " but got BEGIN " & $pemDecoderName(pemCtx))

        readingObj = true
        pemDecoderSetdest(pemCtx, callback, keyCtx)

      of PEM_END_OBJ:
        if readingObj: readingObj = false

      of PEM_ERROR: raise newException(ValueError, "Invalid PEM.")

      else: continue

#[
proc decodeKeyFromPem*(pem: string): JWK =
  var decoderCtx: PemDecoderContext
  pemDecoderInit(addr decoderCtx)

  var bytesRead = 0
  var skCtx: SkeyDecoderContext
  var pkCtx: PkeyDecoderContext
  var keyCtx: pointer = nil
  var keyType: JwkKeyType

  while bytesRead < len(pem):
    bytesRead += pemDecoderPush(
      addr decoderCtx,
      unsafeAddr pem[bytesRead],
      len(pem))

  case pemDecoderEvent(addr decoderCtx):
    of PEM_BEGIN_OBJ:
      if keyCtx != nil:
        raise newException(ValueError, "Invalid PEM: saw a second BEGIN")

      let objName = pemDecoderName(addr decoderCtx)
      if objName == "RSA PRIVATE KEY":
        skeyDecoderInit(addr skCtx)
        keyCtx = addr skCtx
        keyType = JwkKeyType.RsaPrivate
      if objName == "EC PRIVATE KEY":
        skeyDecoderInit(addr skCtx)
        keyCtx = addr skCtx
        keyType = JwkKeyType.EcPrivate
      elif objName == "RSA PUBLIC KEY":
        pkeyDecoderInit(addr pkCtx)
        keyCtx = addr pkCtx
        keyType = JwkKeyType.RsaPublic
      else:
        raise newException(ValueError,
          "Unrecognized PEM object: " & $pemDecoderName(addr decoderCtx))

    of PEM_END_OBJ:
      if keyCtx == nil:
        raise newException(ValueError, "Saw END before BEGIN.")
      elif keyType == JwkKeyType.RsaPrivate:
        return nil

    of PEM_ERROR:
      raise newException(ValueError, "Invalid PEM.")

    else: discard nil

proc decodeRsaPem*(pem: string): JWK =
  var skCtx: SkeyDecoderContext
  skeyDecoderInit(addr skCtx)
  decodePem(
    pem,
    cast[proc(ctx: pointer, data: pointer, dataLen: int) {.bearSslFunc.}](skeyDecoderPush)
    addr skCtx)
  if skeyDecoderLastError(addr skCtx) != 0:
    raise newException(ValueError,
      "Provided PEM could not be decoded as a valid RSA private key.")
]#
