import std/json, std/options, std/strutils, std/unittest

import jwt_full/claims
import ../testdata

suite "jwt_full/claims":

  test "initClaims(b64)":
    let rfcClaims = initJwtClaims(rfc7519_S31_ClaimsB64)
    let sampleClaims = initJwtClaims(sampleJwt.split('.')[1])

    check:
      rfcClaims.iss.isSome
      rfcClaims.iss.get == "joe"
      rfcClaims.exp.isSome
      rfcClaims["http://example.com/is_root"].isSome
      rfcClaims["http://example.com/is_root"].get.kind == JBool
      rfcClaims["http://example.com/is_root"].get.getBool == true
      sampleClaims.sub.isSome
      sampleClaims.sub.get == "abc"
      sampleClaims.iat.isSome
      sampleClaims["name"].isSome
      sampleClaims["name"].get.getStr == "John Doe"

  test "initClaims(json)":
    let jsonClaims = initJwtClaims(parseJson(sampleJwtClaimsDecoded))

    check:
      jsonClaims.sub.isSome
      jsonClaims.sub.get == "abc"
      jsonClaims["name"].isSome
      jsonClaims["name"].get.getStr == "John Doe"
      jsonClaims.iss.isNone
      $jsonClaims == "eyJzdWIiOiJhYmMiLCJuYW1lIjoiSm9obiBEb2UiLCJpYXQiOjE1MTYyMzkwMjJ9"

  test "round trip":
    let rfcClaims1 = initJwtClaims(rfc7519_S31_ClaimsB64)
    let rfcClaims2 = initJwtClaims($rfcClaims1)

    let sampleClaims1 = initJwtClaims(sampleJwt.split('.')[1])
    let sampleClaims2 = initJwtClaims($sampleClaims1)

    check:
      rfcClaims1.iss == rfcClaims2.iss
      rfcClaims1.exp == rfcClaims2.exp
      rfcClaims1.exp == rfcClaims2.exp
      sampleClaims1.sub == sampleClaims2.sub
      sampleClaims1.iat == sampleClaims2.iat
      sampleClaims1["name"].get.getStr == sampleClaims2["name"].get.getStr
