import std/json, std/options, std/strutils, std/unittest

import jwt_full/encoding
import ../testdata

suite "encoding":

  test "b64UrlEncode(int array)":
    check:
      rfc7519_S31_HeaderB64 == b64UrlEncode(rfc7519_S31_HeaderBytes)
      rfc7519_S31_ClaimsB64 == b64UrlEncode(rfc7519_S31_ClaimsBytes)
      rfc7515_A1_HeaderB64 == b64UrlEncode(rfc7515_A1_HeaderBytes)
      rfc7515_A1_ClaimsB64 == b64UrlEncode(rfc7515_A1_ClaimsBytes)

  test "b64UrlEncode(string)":
    let jwtParts = sampleJwt.split('.')

    check:
      b64UrlEncode(sampleJwtHeaderDecoded) == jwtParts[0]
      b64UrlEncode(sampleJwtClaimsDecoded) == jwtParts[1]
      rfc7515_A2_HeaderB64 == b64UrlEncode(rfc7515_A2_HeaderStr)

      # Nim mangles \r\n somehow, causing the below to fail. However, the
      # failure is at the string literal -> string var layer, not the encoding
      # layer)
      # rfc7519_S31_HeaderB64 == b64UrlEncode(rfc7519_S31_HeaderStr)
      # rfc7519_S31_ClaimsB64 == b64UrlEncode(rfc7519_S31_ClaimsStr)

  test "b64UrlDecode(string)":
    let jwtParts = sampleJwt.split('.')

    check:
      sampleJwtHeaderDecoded == b64UrlDecode(jwtParts[0])
      sampleJwtClaimsDecoded == b64UrlDecode(jwtParts[1])
      rfc7515_A2_HeaderStr == b64UrlDecode(rfc7515_A2_HeaderB64)

  test "byteArrToString":
    check:
      byteArrToString(rfc7515_A1_HeaderBytes) == b64UrlDecode(rfc7515_A1_HeaderB64)
