import std/json, std/unittest

import jwt_full

import jwt_full/encoding

import ../testdata

suite "jwt_full/jws":

  let rfc7515_A1_Jwk = initJwk(parseJson(rfc7515_A1_JwkStr))
  let rfc7515_A2_JwkPubKey = initJwk(parseJson(rfc7515_A2_JwkPubStr))
  let rfc7515_A2_JwkPrvKey = initJwk(parseJson(rfc7515_A2_JwkPrvStr))
  let rfc7515_A3_Jwk = initJwk(parseJson(rfc7515_A3_JwkStr))
  let rfc7515_A4_Jwk = initJwk(parseJson(rfc7515_A4_JwkStr))
  let sampleKey = initJwk(parseJson(sampleJwtKey))

  test "HS256 - sign":

    check rfc7515_A1_Jwt == $sign(
      header = initJoseHeader(rfc7515_A1_HeaderB64),
      payload = byteArrToString(rfc7515_A1_ClaimsBytes),
      key = rfc7515_A1_Jwk)

    check sampleJwt == $sign(
      header = initJoseHeader(parseJson(sampleJwtHeaderDecoded)),
      payload = sampleJwtClaimsDecoded,
      key = initJwk(parseJson(sampleJwtKey)))

  test "HS256 - verify":

    validate(
      jws = initJWS(rfc7515_A1_Jwt),
      alg = HS256,
      key = rfc7515_A1_Jwk)

    validate(
      jws = initJWS(sampleJwt),
      alg = HS256,
      key = sampleKey)

  test "HS256 - round trip":

    let payload = "This is a message I want to protect from tampering."

    let jws = sign(
      header = initJoseHeader(%*{ "alg": "HS256" }),
      payload = payload,
      key = sampleKey)

    jws.validate(HS256, sampleKey)

  test "RS256 - sign using JWK":

    check rfc7515_A2_Jwt == $sign(
      header = initJoseHeader(rfc7515_A2_HeaderB64),
      payload = b64UrlDecode(rfc7515_A2_ClaimsB64),
      key = rfc7515_A2_JwkPrvKey)

  test "RS256 - verify using JWK":

    validate(
      jws = initJWS(rfc7515_A2_Jwt),
      alg = RS256,
      key = rfc7515_A2_JwkPubKey)

  test "RS256 - round trip using JWK":

    let jws = sign(
      header = initJoseHeader(%*{"alg":"RS256"}),
      payload = "This is a message I want to protect from tampering.",
      key = rfc7515_A2_JwkPrvKey)

    jws.validate(RS256, rfc7515_A2_JwkPubKey)

#[ TODO
  test "RS256 - sign using PEM":

    #check rfc7515_A2_Jwt == $sign(
    echo $sign(
      header = initJoseHeader(parseJson(sampleJwtRsaHeaderDecoded)),
      payload = $initJwtClaims(parseJson(sampleJwtRsaClaimsDecoded)),
      key = rsPrivateKey)

  test "RS256 - verify using PEM":

    validate(
      jws = initJWS(rfc7515_A2_Jwt),
      alg = RS256,
      key = rsPublicKey)

  test "RS256 - round trip using PEM":

    let jws = sign(
      header = initJoseHeader(%*{"alg":"RS256"}),
      payload = "This is a message I want to protect from tampering.",
      key = rsPrivateKey)

    jws.validate(RS256, rsPublicKey)

  test "ES256 - sign":

    check rfc7515_A3_Jwt == $sign(
      header = initJoseHeader(rfc7515_A3_HeaderB64),
      payload = b64UrlDecode(rfc7515_A3_ClaimsB64),
      key = rfc7515_A3_Jwk)

  test "ES256 - verify":

    validate(
      jws = initJWS(rfc7515_A3_Jwt),
      alg = ES256,
      key = rfc7515_A3_Jwk)

  test "ES256 - round trip":

    let jws = sign(
      header = initJoseHeader(%*{"alg": "ES256"}),
      payload = "This is a message I want to protect from tampering.",
      key = rfc7515_A3_Jwk)

    jws.validate(ES256, rfc7515_A3_Jwk)

  test "ES512 - sign":

    check rfc7515_A4_Jwt == $sign(
      header = initJoseHeader(rfc7515_A4_HeaderB64),
      payload = b64UrlDecode(rfc7515_A4_PayloadB64),
      key = rfc7515_A4_Jwk)

  test "ES512 - verify":

    validate(
      jws = initJWS(rfc7515_A4_Jwt),
      alg = ES512,
      key = rfc7515_A4_Jwk)

  test "ES512 - round trip":

    let jws = sign(
      header = initJoseHeader(%*{"alg": "ES512"}),
      payload = "This is a message I want to protect from tampering.",
      key = rfc7515_A4_Jwk)

    jws.validate(ES512, rfc7515_A4_Jwk)
]#
