import std/json, std/unittest

import jwt_full
import jwt_full/encoding

import ../testdata

suite "jwt_full/jwt":

  let rfc7515_A1_Jwk = initJwk(parseJson(rfc7515_A1_JwkStr))
  let rfc7515_A2_JwkPubKey = initJwk(parseJson(rfc7515_A2_JwkPubStr))
  let rfc7515_A2_JwkPrvKey = initJwk(parseJson(rfc7515_A2_JwkPrvStr))
  let sampleKey = initJwk(parseJson(sampleJwtKey))

  test "sign - HS256":
    check:
      sampleJwt == $createSignedJwt(
        header = initJoseHeader(b64UrlEncode(sampleJwtHeaderDecoded)),
        claims = initJwtClaims(b64UrlEncode(sampleJwtClaimsDecoded)),
        sampleKey)

      rfc7515_A1_Jwt == $createSignedJwt(
        header = initJoseHeader(rfc7515_A1_HeaderB64),
        claims = initJwtClaims(rfc7515_A1_ClaimsB64),
        rfc7515_A1_Jwk)

  test "sign - RS256":
    check:
      rfc7515_A2_Jwt == $createSignedJwt(
        header = initJoseHeader(rfc7515_A2_HeaderB64),
        claims = initJwtClaims(rfc7515_A2_ClaimsB64),
        rfc7515_A2_JwkPrvKey)

  test "validate - HS256":

    initJwt(rfc7515_A1_Jwt).validate(HS256, rfc7515_A1_Jwk, validateTimeClaims = false)
    initJwt(sampleJwt).validate(HS256, sampleKey, validateTimeClaims = false)

  test "validate - RS256":

    initJwt(rfc7515_A2_Jwt).validate(RS256, rfc7515_A2_JwkPubKey, validateTimeClaims = false)
